package Weblog::Channel;

########################################################################
# LOAD MODULES                                                         #
########################################################################
BEGIN {
    use strict;
    use Carp;
    use XML::RSS;
    use Weblog::Item;
    use Weblog::Client;
    use POSIX qw(strftime);
    use vars qw($VERSION $AUTOLOAD @ISA);
    $VERSION = '0.1';
    @ISA = qw();
}

########################################################################
# CLASS INITIALIZATION                                                 #
########################################################################
# list of valid class attributes
my %ok_fields = (
		 'config'      => undef,
		 'name'        => undef,
		 'title'       => undef,
		 'link'        => undef,
		 'description' => undef,
		 'dc'          => undef,
		 'syn'         => undef,
		 'image'       => undef,
		 'textarea'    => undef,
		 'language'    => undef,
		 'date'        => undef,
		 'subject'     => undef,
		 'publisher'   => undef,
		 'creator'     => undef,
		 'type'        => undef,
		 'contributor' => undef,
		 'format'      => undef,
		 'identifier'  => undef,
		 'source'      => undef,
		 'relation'    => undef,
		 'coverage'    => undef,
		 'rights'      => undef,
		 'textinput'   => undef,
		 'image'       => undef,
		 'id'          => {},
		 'items'       => [],
		 'weblog'      => undef,
		 'taxo'        => undef
	       );

sub new {
#**********************************************************************#
    # determine class name
    my $pkg = shift;
    my $class = ref($pkg) || $pkg;
    
    # build new instance of object
    my $self = bless 
    {
        _fields => \%ok_fields
        }, $class;
    
    # initalize variables in object if values were passed
    $self->_initialize(@_);
    
    # return pointer to the new object
    return $self;
#**********************************************************************#
}

sub _initialize {
#**********************************************************************#
    my $self = shift;
    my ($weblog,$name) = @_;

    # ensure that a Weblog object and name were passed
    croak ("Must pass Weblog object and valid channel name")
	unless ($weblog =~ /Weblog/ && exists($weblog->config->channel->{$name}));

    # Copy the hashes instead of using them directly to avoid
    # problems with multiple Weblog::Item objects being used concurrently
    my %template=%ok_fields;
    $self->{_fields} = \%template;

    # assign object attribute values
    $self->weblog($weblog);
    $self->config($weblog->config->channel->{$name});
    $self->name($name);

    # make sure that config and name parameters were passed
    if ($self->config && $self->name) {
	$self->load_channel;
    } else {
	croak("Channel name and configuration required");
    }
#**********************************************************************#
}

sub AUTOLOAD {
#**********************************************************************#
    my $self = shift;
    my $type = ref($self) || croak "$self is not an object\n";
    my $name = $AUTOLOAD;
    $name =~ s/.*://;
    unless (exists $self->{_fields}->{$name}) {
        croak "Unregistered entity: Can't access $name field in object of class $type";
    }
    if (@_) {
        return $self->{_fields}->{$name} = shift;
    } else {
	return $self->{_fields}->{$name};
    }
#**********************************************************************#
}

sub DESTROY {
#**********************************************************************#
    my $self = shift;
    $self = undef;
#**********************************************************************#
}

########################################################################
# METHOD PROTOTYPES                                                    #
########################################################################
# sub add_item ($item, $pos)

# sub concat_text ($maxchars, $string) { }
#**********************************************************************#
# concat_text() concatenates text that is too long. Used for Palm,     #
# Avantgo, and WML outputs.                                            #
#**********************************************************************#

# sub date_time_stamp { }

# sub encode ($text) { }
#**********************************************************************#
# encode() encodes default XML entities.                               #
#**********************************************************************#

# sub list_channels ({%options})                               

# sub load_channel ($channel, {%options})

# sub save_channel ($channel, {%options})

########################################################################
# CLASS METHODS                                                        #
########################################################################

sub add_item {
#**********************************************************************#
# SYNTAX: &add_item;                                                   #
# PARAMS: $item - reference to a Weblog::Item object                   #
#         $pos  - text string: insert or append                        #
# RETURN:                                                              #
#**********************************************************************#
    my ($self,$item,$pos) = @_;
    
    # create a reference to the identifier hash
    $self->{_fields}->{'id'}->{$item->identifier} = $item;

    # insert a new item into the channel
    if ($pos eq 'insert') {
	#warn ("Inserting a new item to channel");
	unshift(@{$self->{_fields}->{'items'}}, $item);
	
    # append a new item into the channel
    } elsif($pos eq 'append') {
	#warn ("Appending new item to channel");
	push(@{$self->{_fields}->{'items'}}, $item);

    # user did not provide an insert or delete string as the second
    # argument to the method
    } else {
	warn ("Invalid position of new item in channel.");
    }
#**********************************************************************#
# END SUBROUTINE: &add_item;                                           #
#**********************************************************************#
}

sub concat_text {
#**********************************************************************#
# SYNTAX: &concat_text($maxchars,$string);                             #
# PARAMS: $maxchars  - maximum number or characters we should allow    #
#         $string    - the original string that we want to concatenate #
# RETURN: $string - the contents of the new string based on the        #
#                   parameters that were passed into the routine       #
#**********************************************************************#
    my ($maxchars, $string) = @_;
    
    # if the length of the original string is greater than the maximum
    # number of allowed characters
    if (length($string) > $maxchars) {
	# cut the string down to the length specified in $maxchars
	# plus an additional 2 characters so we can add .. to the
	# end of the string to indicate the string has been cut.
	$string = substr($string,0,($maxchars - 2));
	$string .= "..";
    }
    
    # return the concatenated string
    return $string;
#**********************************************************************#
# END SUBROUTINE &concat_text($maxchars,$string)                       #
#**********************************************************************#
}

sub date_time_stamp {
#**********************************************************************#
# SYNTAX:         #
# PARAMS:   #
#           #
# RETURN: 1 -          #
#         0 -          #
#**********************************************************************#
    return strftime("%Y-%m-%dT%H:%M",gmtime);
#**********************************************************************#
# END SUBROUTINE &concat_text($maxchars,$string)                       #
#**********************************************************************#
}

sub delete_item {
#**********************************************************************#
# SYNTAX:         #
# PARAMS:   #
#           #
# RETURN: 1 -          #
#         0 -          #
#**********************************************************************#
    my ($self,$identifier) = @_;

    # init local variables
    my $counter = 0;
    my $pos;
    
    # make sure an identifier has been passed
    croak("Must pass a valid item identifer") unless $identifier;

    # remove item from id hash
    delete($self->id->{$identifier});

    # first figure out the array position of the item
    for(my $counter=0; $counter < @{$self->items}; $counter++) {
	my $item = $self->items->[$counter];
	#warn($item->identifier." = $identifier");
	if ($item->identifier eq $identifier) {
	    $pos = $counter;
	    last;
	}
    }

    # if we didn't get a position, the identifier is invalid
    croak ("Invalid identifier: $identifier") unless $pos =~ /\d+/;

    # remove item from the items array
    splice(@{$self->items},$pos,1);
#**********************************************************************#
# END SUBROUTINE &delete_item()                       #
#**********************************************************************#
}

sub encode {
#**********************************************************************#
# SYNTAX: &encode($text);                                              #
# PARAMS: $text - the text that will be encoded                        #
# RETURN: $text - the encoded string                                   #
#**********************************************************************#
    my $self = shift;
    my $text = shift;
    return 0 unless $text;
    $text = HTML::Entities::decode($text);
    $text =~s/\r//gsi;        # delete carraige returns
    $text =~s/\s*\n\s*/ /gsi; # replace returns w/ space
    $text =~ s/[^\w\s\\\/\<\>\&\@\.\,\:\;\'\"\{\}\[\]\-\_\+\=\(\)\!\~\`\@\#\$\%\^\*\?]//g;
    $text = HTML::Entities::encode($text, '\200-\377\'\%\&\<\>\"');
    return $text;
#**********************************************************************#
# END SUBROUTINE &encode($text)                                        #
#**********************************************************************#
}

sub load_channel {
#**********************************************************************#
# SYNTAX: &load_channel;                                               #
# PARAMS:                                                              #
# RETURN:                                                              #
#**********************************************************************#
    # load parameters into memory
    my $self = shift;
    my $config = $self->config;
    my %options = @_;

    # init local variables
    my $rss;
    my @items = ();
    my %id = ();

    # load the channel via the Weblog::Client interface
    if ($config->{'server'}) {

    # load a local RSS file
    } elsif ($config->{'rss1.0'}) {
	my $rssfile = $config->{'rss1.0'};

	# make sure the RSS file exists
	if(! -e $rssfile) {
	    croak ("RSS File $rssfile does not exist");	
	}
	
	# parse the RSS file
	$rss = new XML::RSS;
	eval { 
	    $rss->parsefile($rssfile) 
	    };
	
	# exit if there was an error parsing the RSS file
	croak $@ if $@;
    }

    # create channel attributes
    $self->title($rss->{'channel'}->{'title'});
    $self->link($rss->{'channel'}->{'link'});
    $self->description($rss->{'channel'}->{'description'});
    $self->date($rss->{'channel'}->{'dc'}->{'date'});
    $self->subject($rss->{'channel'}->{'dc'}->{'subject'});
    $self->publisher($rss->{'channel'}->{'dc'}->{'publisher'});
    $self->rights($rss->{'channel'}->{'dc'}->{'rights'});
    $self->image($rss->image);
    $self->textinput($rss->textinput);

    $self->creator($rss->{'channel'}->{'dc'}->{'creator'});
    $self->publisher($rss->{'channel'}->{'dc'}->{'publisher'});
    $self->subject($rss->{'channel'}->{'dc'}->{'subject'});
    $self->contributor($rss->{'channel'}->{'dc'}->{'contributor'});
    $self->type($rss->{'channel'}->{'dc'}->{'type'});
    $self->format($rss->{'channel'}->{'dc'}->{'format'});
    $self->identifier($rss->{'channel'}->{'dc'}->{'identifier'});
    $self->source($rss->{'channel'}->{'dc'}->{'source'});
    $self->relation($rss->{'channel'}->{'dc'}->{'relation'});
    $self->coverage($rss->{'channel'}->{'dc'}->{'coverage'});
    $self->rights($rss->{'channel'}->{'dc'}->{'rights'});

    # taxo
    $self->taxo($rss->{'channel'}->{'taxo'});

    # get language element
    if ($rss->{'channel'}->{'language'}) {
	#print $rss->{'channel'}->{'language'}."\n";
	$self->language($rss->{'channel'}->{'language'});
    } elsif($rss->{'channel'}->{'dc'}->{'language'}) {
	#print $rss->{'channel'}->{'dc'}->{'language'}."\n";
	$self->language($rss->{'channel'}->{'dc'}->{'language'});
    } elsif ($options{'language'}) {
	#print $options{'language'}."\n";
	$self->language($options{'language'});
    } 

    # create new instances of Weblog::Item for each RSS item
    foreach my $rssitem (@{$rss->{'items'}}) {
	#print Dumper($rssitem);
	my $item = new Weblog::Item(%$rssitem);
	$item->channel($self);
	$id{$item->identifier} = $item;
	#print $id{$item->identifier};
	push(@items, $item);
    }

    $self->id(\%id);
    $self->items(\@items);
#**********************************************************************#
# END SUBROUTINE: &load_channel;                                       #
#**********************************************************************#
}

sub move_item {
#**********************************************************************#
# SYNTAX:         #
# PARAMS:   #
#           #
# RETURN: 1 -          #
#         0 -          #
#**********************************************************************#
    
#**********************************************************************#
# END SUBROUTINE &move_item()                       #
#**********************************************************************#
}

sub save {
#**********************************************************************#
# SYNTAX: &save;                                                       #
# RETURN: 1 - channel was successfully saved                           #
#         0 - channel was not saved                                    #
#**********************************************************************#
    my $self = shift;

    # create a new instance if the XML::RSS class
    my $rss = new XML::RSS (version => '1.0');
 
    # check for required elements
    if (!($self->title)) {
	croak ("Channel must contain a title element");
    } elsif(!($self->link)) {
	croak ("Channel must contain a link element");
    } elsif(!($self->language)) {
	croak ("Channel must contain a language element");
    }

    # load up dc hash with channel values
    $self->{_fields}->{'dc'}->{'creator'} = $self->creator;
    $self->{_fields}->{'dc'}->{'publisher'} = $self->publisher;
    $self->{_fields}->{'dc'}->{'subject'} = $self->subject;
    $self->{_fields}->{'dc'}->{'contributor'} = $self->contributor;
    $self->{_fields}->{'dc'}->{'date'} = $self->date_time_stamp;
    $self->{_fields}->{'dc'}->{'type'} = $self->type;
    $self->{_fields}->{'dc'}->{'format'} = $self->format;
    $self->{_fields}->{'dc'}->{'identifier'} = $self->identifier;
    $self->{_fields}->{'dc'}->{'source'} = $self->source;
    $self->{_fields}->{'dc'}->{'relation'} = $self->relation;
    $self->{_fields}->{'dc'}->{'coverage'} = $self->coverage;
    $self->{_fields}->{'dc'}->{'rights'} = $self->rights;

    # channel info
    $rss->channel(title          => $self->encode($self->title),
		  link           => $self->encode($self->link),
		  description    => $self->encode($self->description),
		  language       => $self->encode($self->language),
		  dc             => $self->dc,
		  taxo           => $self->taxo
		  );

    # image
    $rss->image(%{$self->image});  

    # textinput
    $rss->textinput(%{$self->textinput}); 

    # add the items to the XML::RSS object
    foreach my $item (@{$self->items}) {
	# make sure the item has all required elements.
	# If it doesn't ignore it.
	if (!($item->title)) {
	    croak ("Item must contain a title");
	} elsif (!($item->link)) {
	    croak ("Item must contain a link");
	}

	# assign taxo element if it exists
	my $taxo;
	if ($item->taxo) {
	    $taxo = $item->taxo;
	}

	$rss->add_item (
			title       => $self->encode($item->title),
			link        => $self->encode($item->link),
			description => $self->encode($item->description),
			dc => {
			    'subject'    => $self->encode($item->subject),
			    'creator'    => $self->encode($item->creator),
			    'date'       => $self->encode($item->date),
			    'source'     => $self->encode($item->source),
			    'format'     => $self->encode($item->format),
			    'identifier' => $self->encode($item->identifier),
			    'type'       => $self->encode($item->type)
			    },
			taxo => $taxo
			);
    }

    # save the RSS file in whatever RSS versions have been specified.
    # RSS 1.0 is required.
    if ($self->config->{'rss0.9'}) {
	$rss->{output} = '0.9'; 
	$rss->save($self->config->{'rss0.9'});
    }
    if ($self->config->{'rss0.91'}) {
	$rss->{output} = '0.91'; 
	$rss->save($self->config->{'rss0.91'});
    }
    if ($self->config->{'rss1.0'}) {
	$rss->{output} = '1.0'; 
	$rss->save($self->config->{'rss1.0'});
    } else {
	croak ("RSS 1.0 output is required");
    }
#**********************************************************************#
# END SUBROUTINE: &save;                                               #
#**********************************************************************#
}

1;
__END__
