package Weblog::Output::Template;

########################################################################
# LOAD MODULES                                                         #
########################################################################
BEGIN {
    use strict;
    use Carp;
    use HTML::Template;
    use vars qw($VERSION $AUTOLOAD @ISA);
    $VERSION = '0.1';
    @ISA = qw();
}

########################################################################
# CLASS INITIALIZATION                                                 #
########################################################################
# list of valid class attributes
my %ok_fields = (
		 channel => undef,
		 'format'  => {
		     'html' => {
			 'template'  => 'html.tmpl'
			 },
		     'vxml' => {
			 'template'  => 'vxml.tmpl'
			 },
		     'palm' => {
			 'extension' => 'html',
			 'template'  => 'palm.tmpl',
			 'story'     => 'palm_story.tmpl'
			 },
		     'wml' => {
			 'extension' => 'wml',
			 'template'  => 'wml.tmpl',
			 'story'     => 'wml_story.tmpl'
			 }, 
		     'avantgo' => {
			 'extension' => 'html',
			 'template'  => 'avantgo.tmpl',
			 'story'     => 'avantgo_story.tmpl'
			 }
		 }
	       );

sub new {
#**********************************************************************#
    # determine class name
    my $pkg = shift;
    my $class = ref($pkg) || $pkg;
    
    # build new instance of object
    my $self = bless 
    {
        _fields => \%ok_fields
        }, $class;
    
    # initalize variables in object if values were passed
    $self->_initialize(@_);
    
    # return pointer to the new object
    return $self;
#**********************************************************************#
}

sub _initialize {
#**********************************************************************#
    my ($self, $channel) = @_;

    # Copy the hashes instead of using them directly to avoid
    # problems with multiple objects being used concurrently
    my %template=%ok_fields;
    $self->{_fields} = \%template;

    # ensure that a channel object and type were passed
    croak ("Must pass Weblog::Channel object")
	unless ($channel =~ /Weblog::Channel/);

    $self->channel($channel);
    $self->format($ok_fields{'format'});
#**********************************************************************#
}

sub AUTOLOAD {
#**********************************************************************#
    my $self = shift;
    my $type = ref($self) || croak "$self is not an object\n";
    my $name = $AUTOLOAD;
    $name =~ s/.*://;
    unless (exists $self->{_fields}->{$name}) {
        croak "Unregistered entity: Can't access $name field in object of class $type";
    }
    if (@_) {
        return $self->{$name} = shift;
    } else {
	return $self->{$name};
    }
#**********************************************************************#
}

sub DESTROY {
#**********************************************************************#
    my $self = shift;
    $self = undef;
#**********************************************************************#
}

########################################################################
# METHOD PROTOTYPES                                                    #
########################################################################

# sub output ($format) { }


########################################################################
# CLASS METHODS                                                        #
########################################################################

sub output {
#**********************************************************************#
# SYNTAX: &output($format);                                            #
# PARAMS:                    #
#                                 #
# RETURN:                                                              #
#**********************************************************************#
    my ($self,$format) = @_;

    # init local variables
    my $config;
    my $weblog;
    my $channel;

    # exit if no value has been defined for the format element in the
    # config.xml file for the channel
    if(!$self->channel->config->{$format}) {
	warn ("<$format> element not detected in config.xml");
	return 0;
    } else {
	$config = $self->channel->config;
	$weblog = $self->channel->weblog->config; 
	$channel = $self->channel;
    }
    
    # build the name of the template using the <tmpl_dir> element and
    # format filename as specified in the config.xml file.
    my $tmpl = $config->{'tmpl_dir'} . "/" . $self->format->{$format}->{'template'};

    # make sure the template exists
    croak ("$format template $tmpl does not exist.") unless -e $tmpl;

    # create a new instance of the HTML::Template module using the 
    # format template for the channel. Each channel has its own template
    # directory with one or more templates for each output format.
    # We turn off die_on_bad_params so that the program won't croak
    # if users remove one of the template parameters from the template
    # file.
    my $template = HTML::Template->new(filename          => $tmpl,
				       die_on_bad_params => 0);

    # open the format output file for write or croak
    open(TMPL,">$config->{$format}") ||
	croak("Cannot write to $config->{$format}: $!");
    
    # We need to build an array that will be populated by RSS
    # item values and outputted via the HTML::Template template
    # file
    my @tmpl_array = ();
    
    # build the data structure for HTML::Template
    my $count = 1;

    # loop over each RSS item
    foreach my $item (@{$channel->items}) {
	# skip the item unless there's a title and link
	next unless $item->title && $item->link;

	# exit the loop if we've reached the item limit specifed
	# in the <max_items> element in the config.xml file
	last unless $count <= $config->{max_items}->{$format};

	# now we build the hash that contains the item contents and
	# push it the the array
	my %row = ();

	# Build story files for formats that have an index file and
	# one file for each item
	if ($self->format->{$format}->{'story'}) {
	    # need to build story files
	    $row{'TITLE'}       = $channel->concat_text($item->title);
	    $row{'DESCRIPTION'} = $channel->concat_text($item->description);
	    $row{'LINK'}        = "$count.".$self->format->{$format}->{'extension'};

	    # build story file for the rss item based on the template
	    # directory and filename per the config.xml file.
	    my $story_tmpl = $config->{'tmpl_dir'} . "/" . $self->format->{$format}->{'story'};

	    # create a new instance of the HTML::Template module using the
	    # story template
	    my $story = HTML::Template->new(filename      => $story_tmpl,
					die_on_bad_params => 0);

	    # open the output file or croak
	    my $story_dir = $format."_story";
	    my $story_file = $config->{$story_dir}."/$count.".$self->format->{$format}->{'extension'};
	    open(STORY,">$story_file") || 
		croak("Cannot write to $story_file");
	
	    # set the story template values based on the rss item
	    $story->param(STORY_TITLE => $row{'TITLE'});
	    $story->param(DESCRIPTION => $row{'DESCRIPTION'});
	    $story->param(LINK        => $item->link);
	    
	    # print the story file
	    print STORY $story->output();
	    close(STORY);
	    
        # regular one file output
	} else {
	    $row{'TITLE'}       = $item->title;
	    $row{'DESCRIPTION'} = $item->description;
	    $row{'LINK'}        = $item->link;
	}
	push(@tmpl_array,\%row);
	$count++;
    }
    
    # set item array in template
    $template->param(RSS_LOOP => \@tmpl_array);
    
    # set the title in the template
    $template->param(FULLNAME    => $channel->title);
    $template->param(DESCRIPTION => $channel->description);
    
    # generate the output file
    print TMPL $template->output();
    close(TMPL);

    # output file was written successfully
    return 1;
#**********************************************************************#
# END SUBROUTINE: &output($format);                                    #
#**********************************************************************#
}

sub outputall {
#**********************************************************************#
# SYNTAX: &outputall;                                                  #
# PARAMS:                                                              #
# RETURN:                                                              #
#**********************************************************************#
    my $self = shift;
    foreach my $format (keys(%{$self->format})) {
	$self->output($format);	
    }
#**********************************************************************#
# END SUBROUTINE: &outputall;                                          #
#**********************************************************************#
}

1;
